# This file contains R scripts used for preprocessing and visualizing the sponge microbiome data.
# Scripts for running anuran can be found in globalsponges.py.

setwd("~/anuran_supplements/globalsponge_data")
library(phyloseq)

prevFilter = function(phy, prev){
  prev0 = apply(X = otu_table(phy),
                MARGIN = ifelse(taxa_are_rows(phy), yes = 1, no = 2),
                FUN = function(x){sum(x > 0)})
  prevalenceThreshold = prev * nsamples(phy)
  nonrares = prune_taxa((prev0 > prevalenceThreshold), phy)
  # making a phylo of rare taxa does not work if there are no rare taxa
  rares = prune_taxa((prev0 < prevalenceThreshold), phy)
  rares = merge_taxa(rares, taxa_names(rares))
  otus = data.frame(otu_table(nonrares))
  otus = rbind(otus, data.frame(otu_table(rares)))
  tax = data.frame(tax_table(nonrares), stringsAsFactors = FALSE)
  tax = rbind(tax, c("Bacteria", rep(NA, 6)))
  rownames(tax)[length(rownames(tax))] = taxa_names(rares)
  colnames(otus) <- rownames(sample_data(phy))  # checked manually if this is correct; data.frame adds an X for every name
  newphy = phyloseq(otu_table(otus, taxa_are_rows = TRUE), sample_data(phy), tax_table(as.matrix(tax)))
  return(newphy)
}

# link: https://academic.oup.com/gigascience/article/6/10/gix077/4082886#supplementary-data
# samples from Giga Science Deblur table
data <- import_biom('QIIME-output.biom', sep='')
data <- tax_glom(data, taxrank='Rank6', NArm=FALSE)
saveRDS(data, 'QIIME_rank6.RDS')
meta <- read.csv("sample.metadata.tsv", header=TRUE, sep='\t') # sample table from Giga Science publication
# need to add metadata to phyloseq object
# last couple of samples did not have deblur names updated
meta <- meta[meta$sample_name_qiime_deblur %in% sample_names(data),]
rownames(meta) <- meta$sample_name_qiime_deblur
sample_data(data) <- meta

# first remove samples with extremely low counts
lowcounts <- sample_sums(data)
names(lowcounts) <- sample_names(data)
lowcounts <- lowcounts[sample_sums(data) < 1000]
phylo <- prune_samples((!sample_names(data) %in% names(lowcounts)), data)
#phylo <- tax_glom(phylo, taxrank="Rank6", NArm = FALSE)
set.seed(7)
# rarefy to even depth
phylo <- rarefy_even_depth(phylo)

# 20% prevalence filter
phylo <- prevFilter(phylo, 0.2)
meta <- data.frame(sample_data(phylo))

setwd("host_order")
# to run network inference, the data files are first preprocessed
# these files are saved in a folder named host_order
names = c()
networks = c()
for (biome in unique(meta$scientific_order)){
  if (!is.na(biome)){
    subphylo <- subset_samples(phylo, scientific_order == biome)
    subphylo <- prevFilter(subphylo, 0.2)
    print(c(biome, nsamples(subphylo)))
    if (nsamples(subphylo) > 50){
      names <- c(names, biome)
      otus <- data.frame(otu_table(subphylo))
      tax <- data.frame(as.matrix(tax_table(subphylo))[,1:6])
      rownames(otus) <- paste("c", rownames(otus), sep="")
      rownames(tax) <- paste("c", rownames(tax), sep="")
      # remove a trailing space
      # edit files for FlashWeave
      name = gsub(" ", "", biome, fixed = TRUE)
      cat("#otu \t", file = paste(name, 'otu.tsv', sep='_')) 
      write.table(otus, file = paste(name, 'otu.tsv', sep='_'), append = TRUE, quote=FALSE, sep="\t")
      cat("#otu \t", file = paste(name, 'otu.txt', sep='_')) 
      write.table(otus, file = paste(name, 'otu.txt', sep='_'), append = TRUE, quote=FALSE, sep="\t")
      cat("#otu \t", file = paste(name, 'tax.txt', sep='_')) 
      write.table(tax, file = paste(name, 'tax.txt', sep='_'), append = TRUE, quote=FALSE, sep="\t")
    }
  }
}

# the globalsponges.py contains scripts for running CoNet on these files
# the flashweave_script.jl contains scripts for running Flashweave on these files
# the raw networks are further processed in Python to add metadata etc

# get network size
network_sizes <- c()
network_path <- list.files('conet')
for (file in network_path){
  network <- read.graph(format= 'graphml', file=paste('conet', file, sep='/'))
  network_sizes = c(network_sizes, length(E(network)))
}
median(network_sizes)
min(network_sizes)
max(network_sizes)

##### visualizing anuran outputs
data <- read.csv("globalsponge_core_set_differences.csv", stringsAsFactors = FALSE)

# # range of core size from 0.05 to 0.3 and prev from 0.1 to 1
names <- c("Input", "Degree", "Random",
           "Degree size: 0.2 prev:0.5", "Random size: 0.2 prev:0.5",
           "Degree size: 0.2 prev:0.2", "Random size: 0.2 prev:0.2")
data <- data[data$Network %in% names,]
data$Network[data$Network == "Degree size: 0.2 prev:0.5"] = "Degree \nsize: 0.2 \nprev:0.5\n"
data$Network[data$Network == "Random size: 0.2 prev:0.5"] = "Random \nsize: 0.2 \nprev:0.5\n"
data$Network[data$Network == "Degree size: 0.2 prev:0.2"] = "Degree \nsize: 0.2 \nprev:0.2\n"
data$Network[data$Network == "Random size: 0.2 prev:0.2"] = "Random \nsize: 0.2 \nprev:0.2\n"
data$Network <- factor(data$Network, levels=c("Input", "Degree", "Random",
                                              "Degree \nsize: 0.2 \nprev:0.2\n", "Random \nsize: 0.2 \nprev:0.2\n",
                                              "Degree \nsize: 0.2 \nprev:0.5\n", "Random \nsize: 0.2 \nprev:0.5\n"))

# rename intervals
data$Interval[data$Interval == "0->0.2"] <- '1'
data$Interval[data$Interval == "0.2->0.3"] <- '2 -> 3'
data$Interval[data$Interval == "0.3->0.4"] <- '3 -> 4'
data$Interval[data$Interval == "0.4->0.5"] <- '4 -> 5'
data$Interval[data$Interval == "0.5->0.6"] <- '5 -> 6'
data$Interval[data$Interval == "0.6->0.7"] <- '6 -> 7'
data$Interval[data$Interval == "0.7->0.8"] <- '7 -> 8'
data$Interval[data$Interval == "0.8->0.9"] <- '8 -> 9'
data$Interval[data$Interval == "0.9->1"] <- '9 -> 10'
data$Interval[data$Interval == "1->1"] <- '10'
data$Interval <- factor(data$Interval, levels=c('1', '2 -> 3', '3 -> 4', '4 -> 5', '5 -> 6', '6 -> 7', '7 -> 8', '8 -> 9', '9 -> 10', '10'))
data <- plyr::ddply(data, c("Network", "Interval"), plyr::summarise, mean=mean(Set.size), sd=sd(Set.size), sem=(sd(Set.size)/sqrt(length(Set.size))))
# omit difference, since this makes the figure unclear
data <- data[!data$Interval %in% c('1', '2 -> 3'),]

# Use paired colour palette, but set observed data to black
cols <- RColorBrewer::brewer.pal(6, "Paired")
cols <- c("#000000", cols)

ggplot(data, aes(x=Interval, y=mean, colour=Network)) +
  geom_line(aes(x=as.numeric(Interval), y=mean, linetype=Network, size=Network)) + 
  geom_errorbar(aes(x=as.numeric(Interval), ymin=mean-sem, ymax=mean+sem), width=.1, show.legend=FALSE)+
  theme_minimal() +   
  scale_x_continuous(name = "Difference of intersections",
                     breaks=as.numeric(data$Interval),
                     labels = data$Interval)+
  theme(strip.text.x = element_blank(), panel.grid.minor.x = element_blank(), axis.text.x = element_text(angle=70, hjust=1)) + 
  scale_color_manual(values=cols) + 
  scale_linetype_manual(name='Network', breaks=c('Input', 'Degree', 'Random',
                                                 'Degree \nsize: 0.2 \nprev:0.2\n',
                                                 'Random \nsize: 0.2 \nprev:0.2\n',
                                                 'Degree \nsize: 0.2 \nprev:0.5\n',
                                                 'Random \nsize: 0.2 \nprev:0.5\n'),
                        values=c('solid', 'dashed', 'dashed', 'dotted', 'dotted', 'dotted', 'dotted'), drop=FALSE) + 
  scale_size_manual(name='Network', breaks=c('Input', 'Degree', 'Random',
                                             'Degree \nsize: 0.2 \nprev:0.2\n',
                                             'Random \nsize: 0.2 \nprev:0.2\n',
                                             'Degree \nsize: 0.2 \nprev:0.5\n',
                                             'Random \nsize: 0.2 \nprev:0.5\n'),
                    values=c(1.5, 1, 1, 1, 1, 1, 1)) +
  labs(y='Set size', x='Difference of intersections')
ggsave("../Figures/sponge_sets.pdf", width = 4, height=5, units="in")
Sys.sleep(5)

data <- read.csv("globalsponge_core_centralities.csv", stringsAsFactors = FALSE)
names <- c("Input", "Degree", "Random")
data <- data[data$Network %in% names,]
data$Network <- factor(data$Network, levels=c("Input", "Degree", "Random"))
#biom <- import_biom('QIIME-output.biom')
data$tax <- rep('NA', length(rownames(data)))
tax <- as.matrix(tax_table(biom))
for (i in 1:length(rownames(data))){
  nodename <- substr(data$Node[i], 2, nchar(data$Node[i]))
  if (nodename != 'in'){
    taxrow <- tax[which(rownames(tax)==nodename),]
    # get highest level assignment
    found_tax = FALSE
    j = 6
    while (!found_tax && j > 0){
      taxval <- taxrow[,j]
      if (!is.na(taxval)){
        if (nchar(taxval) > 3){
          taxval <- substr(taxval, 4, nchar(taxrow[,j]))
          data$tax[i] <- taxval
          found_tax = TRUE
        }
        
        else{
          j = j - 1
        }}
      else{ 
        j = j - 1}
    }
  }
}

library(ggrepel)
data <- data[data$Centrality %in% c('Betweenness'),]
data <- data[data$Network %in% c('Input', 'Random', 'Degree'),]
stats <- read.csv("globalsponge_core_centrality_stats.csv", stringsAsFactors=FALSE)
stats <- stats[stats$Comparison %in% c('Random', 'Degree'),]
stats <- stats[stats$P < 0.3,]
data <- data[data$Node %in% stats$Node,]
# set data$tax to NA for random
data$tax[data$tax == 'NA'] <- 'Bin'
data$tax[data$Network == 'Random'] <- NA
data$tax[data$Network == 'Degree'] <- NA
ggplot(data, aes(x=Lower.limit, y=Upper.limit, colour=Network)) + 
  geom_point(alpha=0.5, size=2) + facet_grid(~Node) + 
  scale_y_continuous(limits=c(0.3, 1)) + 
  scale_x_continuous(limits=c(0, 1)) + 
  theme_minimal() + theme(axis.text.x = element_text(angle=90, hjust=1)) + scale_color_viridis_d() + 
  geom_label_repel(label=data$tax, nudge_y=-0.1, size=2.9) + 
  xlab("Lower limit of 95% confidence interval") + ylab("Upper limit") + theme(strip.text.x = element_blank())
ggsave("../Figures/sponges_topcentralities.pdf", width = 6.5, height=4, units="in")
Sys.sleep(5)

# annotate stats table with taxonomy
#biom <- import_biom("QIIME-output.biom")
tax <- data.frame(as.matrix(tax_table(biom))[,1:6], stringsAsFactors = FALSE)
centrality_stats <- read.csv("globalsponge_core_centrality_stats.csv", stringsAsFactors=FALSE)
stats <- centrality_stats[centrality_stats$Comparison %in% c('Random', 'Degree'),]
stats$Phylum <- stats$Node
stats$Class <- stats$Node
stats$Order <- stats$Node
stats$Family <- stats$Node
stats$Genus <- stats$Node
for (i in 1:length(rownames(stats))){
  if (substring(stats$Node[i], 1, 1) == 'c'){
    # removes c prefix necessary for CoNet
    name <-substring(stats$Node[i], 2)
    taxa <- tax[rownames(tax) == name,]
    stats[i,9:13] <- taxa[2:6]
  }
}

