# This file contains R scripts used for preprocessing and visualizing the gut microbiome data.
# Scripts for running fastLSA and anuran can be found in gutmicrobiome.txt.
setwd("~/anuran_supplements/gutmicrobiome_data")
library(igraph)
library(ggplot2)
library(ggpubr)

# These files contain information from human subjects,
# and could therefore not be included with the supplementary files.
# They are available upon request from the corresponding author of the original publication. 
rmp
tabs
rmptabs 

##### fastLSA
# tables should be tab-delimited
# each set of columns is a set of replicates at 1 time point
# so: time point 1, replicate 1, time point 2, replicate 2
# we don't have replicates, so we make a single time series network for each participant. 

# 50% participant prevalence
taxcounts <- data.frame(matrix(ncol=1, nrow=length(colnames(rmptabs))))
colnames(taxcounts) <- 'V1'
taxcounts$V1 <- 0
rownames(taxcounts) <- colnames(rmptabs)
taxcounts$names <- rownames(taxcounts)
for (i in 1:length(rmp)){
  data <- na.omit(rmp[[i]])
  data <- data[,colSums(data) > 0]
  for (k in 1:length(rownames(taxcounts))){
    if (rownames(taxcounts)[[k]] %in% colnames(data)){
      taxcounts[k,1] <- taxcounts[k,1] + 1
    }
  }
}
taxcounts <- taxcounts[taxcounts$V1 >= 10,]
taxlist <- taxcounts$names

lsa_names <- c()
# elsa preprocessing
# scale by cycle length
for (i in 1:length(rmp)){
  data <- t(seqtime::normalize(t(rmp[[i]])))
  metadata <- tabs[[i]]
  metadata$Participant <- i
  rownames(data) <- rownames(metadata)
  data <- na.omit(data)
  data <- data[,colMeans(data) >0]
  metadata <- metadata[rownames(metadata) %in% rownames(data),]
  metadata <- metadata[metadata$Scaled_Cycle_through <= 1.0,]
  names <- c(1:21)
  metadata <- metadata[!rownames(metadata) %in% names,]
  data <- data[rownames(data) %in% rownames(metadata),]
  metadata$Discrete_Cycle <- as.integer(cut(metadata$Scaled_Cycle, seq(0, 1.0, 0.0355), right=FALSE, labels=c(1:28)))
  metadata$Discrete_Cycle[metadata$Scaled_Cycle == 1.0] = 28
  # only select one observation per bin
  previous_cycle <- 0
  previous_id <- 0
  selected <- c()
  k = 0
  for (j in 1:length(metadata$Discrete_Cycle)){
    cycle <- metadata$Discrete_Cycle[[j]]
    if (previous_cycle == cycle){
      selected <- c(selected, j)
      k = k +1
    }
    previous_cycle <- cycle
  }
  # also need to remove double measurements 
  if (!is.null(selected)){
    metadata <- metadata[-selected,]
  }
  metadata <- metadata[!rownames(metadata) %in% c('4.1'),]
  data <- data[rownames(data) %in% rownames(metadata),]
  rownames(metadata) <- paste(metadata$Discrete_Cycle, metadata$Participant, sep="c")
  rownames(data) <- rownames(metadata)
  rownames(data) <- rownames(metadata)
  data <- data[,colnames(data) %in% taxlist]
  # we make a dataframe to store the otus and keep missing values
  elsa <- data.frame(matrix(nrow=length(colnames(data)), ncol=28))
  colnames(elsa) <- paste(rep(1:28), i, sep="c")
  rownames(elsa) <- colnames(data)
  for (j in 1:length(colnames(elsa))){
    sample <- t(data[rownames(data) %in% colnames(elsa)[[j]],])
    if (dim(sample)[[1]] == 1){
      elsa[,j] <- as.vector(sample)
    }
    else {
      elsa[,j] <- 'na'
    }
  }
  write.table(elsa, paste("mnm_core50_", i, ".txt", sep=""), row.names=FALSE, col.names=FALSE, sep="\t", quote = FALSE)
  # get list with names / indices for fastlsa parsing
  filenames <- rownames(elsa)
  lsa_names[[i]] <- filenames
}

### preprocess raw fastLSA outputs
network_sizes <- c()
files <- list.files('fastlsa/raw output')
for (i in 1:length(lsa_names)){
    setwd('fastlsa/raw output')
    network <- read.table(files[[i]], header=TRUE)
    # get network id
    num <- as.integer(strsplit(strsplit(files[[i]], "_")[[1]][3], ".txt"))
    # construct edge list
    edges <- as.matrix(network[,c(1:5)])
    for (j in 1:dim(edges)[[1]]){
      edges[j,1] <- lsa_names[[num]][as.integer(edges[j,1])+1]
      edges[j,2] <- lsa_names[[num]][as.integer(edges[j,2])+1]
    }
    # add local similarity score as edge weights
    graph <- graph_from_edgelist(edges[,1:2], directed=FALSE)
    graph <- set_edge_attr(graph, 'weight', value=as.numeric(edges[,3]))
    graph <- set_edge_attr(graph, 'lag', value=edges[,4])
    graph <- set_edge_attr(graph, 'p', value=edges[,5])
    setwd('..')
    name = gsub('.txt', '.graphml', files[[i]])
    write_graph(graph, file=name, format='graphml')
    print(length(E(graph)))
    network_sizes = c(network_sizes, length(E(graph)))
}
median(network_sizes)
min(network_sizes)
max(network_sizes)

### visualize anuran results
data <- read.csv("fastlsa_core_set_differences.csv", stringsAsFactors = FALSE)
CANsize <- data[data$Interval == '0.2->0.3',]
print(CANsize[CANsize$Network == 'Input',])
print(median(CANsize[CANsize$Network == 'Random',]$Set.size))
print(median(CANsize[CANsize$Network == 'Degree',]$Set.size))
print(median(CANsize[CANsize$Network == "Degree size: 0.2 prev:0.1",]$Set.size))
print(median(CANsize[CANsize$Network == "Random size: 0.2 prev:0.1",]$Set.size))
print(median(CANsize[CANsize$Network == "Degree size: 0.2 prev:0.5",]$Set.size))
print(median(CANsize[CANsize$Network == "Random size: 0.2 prev:0.5",]$Set.size))

# rename intervals so instead of fractions, we get absolute network number (here 20 networks)
data$Interval[data$Interval == "0->0.1"] <- '1'
data$Interval[data$Interval == "0.1->0.2"] <- '2 -> 4'
data$Interval[data$Interval == "0.2->0.3"] <- '4 -> 6'
data$Interval[data$Interval == "0.3->0.5"] <- '6 -> 10'
data$Interval[data$Interval == "0.5->1"] <- '10 -> 20'
data$Interval[data$Interval == "1->1"] <- '20'

data$Interval <- factor(data$Interval, levels=c('1', '2 -> 4', '4 -> 6', '6 -> 10', '10 -> 20', '20'))
# range of core size from 0.05 to 0.3 and prev from 0.1 to 1
names <- c("Input", "Degree", "Random",
           "Degree size: 0.2 prev:0.1", "Random size: 0.2 prev:0.1",
           "Degree size: 0.2 prev:0.5",  "Random size: 0.2 prev:0.5")
data <- data[data$Network %in% names,]
# rename label so legend fits better in screen
data$Network[data$Network == "Degree size: 0.2 prev:0.1"] = "Degree \nsize: 0.2 \nprev:0.1\n"
data$Network[data$Network == "Random size: 0.2 prev:0.1"] = "Random \nsize: 0.2 \nprev:0.1\n"
data$Network[data$Network == "Degree size: 0.2 prev:0.5"] = "Degree \nsize: 0.2 \nprev:0.5\n"
data$Network[data$Network == "Random size: 0.2 prev:0.5"] = "Random \nsize: 0.2 \nprev:0.5\n"
data$Network <- factor(data$Network, levels=c("Input", "Degree", "Random",
                                              "Degree \nsize: 0.2 \nprev:0.1\n", "Random \nsize: 0.2 \nprev:0.1\n",
                                              "Degree \nsize: 0.2 \nprev:0.5\n", "Random \nsize: 0.2 \nprev:0.5\n"))
data <- plyr::ddply(data, c("Network", "Interval"), plyr::summarise, mean=mean(Set.size), sd=sd(Set.size), sem=(sd(Set.size)/sqrt(length(Set.size))))
# omit difference, since this makes the figure unclear
data <- data[data$Interval != 1,]

# Use paired colour palette, but set observed data to black
cols <- RColorBrewer::brewer.pal(6, "Paired")
cols <- c("#000000", cols)
ggplot(data, aes(x=Interval, y=mean, colour=Network)) +
  geom_line(aes(x=as.numeric(Interval), y=mean, linetype=Network, size=Network)) + 
  geom_errorbar(aes(x=as.numeric(Interval), ymin=mean-sem, ymax=mean+sem), width=.1, show.legend=FALSE)+
  theme_minimal() + theme(axis.text.x = element_text(angle=70, hjust=1)) + 
  theme(strip.text.x = element_blank(), panel.grid.minor.x = element_blank()) + 
  scale_x_continuous(name = "Difference of intersections",
                     breaks=as.numeric(data$Interval),
                     labels = data$Interval)+
  scale_color_manual(values= cols) + 
  scale_linetype_manual(name='Network', breaks=c('Input', 'Degree', 'Random',
                                                 'Degree \nsize: 0.2 \nprev:0.1\n',
                                                 'Random \nsize: 0.2 \nprev:0.1\n',
                                                 'Degree \nsize: 0.2 \nprev:0.5\n',
                                                 'Random \nsize: 0.2 \nprev:0.5\n'),
                        values=c('solid', 'dashed', 'dashed', 'dotted', 'dotted', 'dotted', 'dotted'), drop=FALSE) + 
  scale_size_manual(name='Network', breaks=c('Input', 'Degree', 'Random',
                                             'Degree \nsize: 0.2 \nprev:0.1\n',
                                             'Random \nsize: 0.2 \nprev:0.1\n',
                                             'Degree \nsize: 0.2 \nprev:0.5\n',
                                             'Random \nsize: 0.2 \nprev:0.5\n'),
                    values=c(1.5, 1, 1, 1, 1, 1, 1)) +
  labs(y='Set size')
ggsave("../Figures/gutmicrobiome_sets.pdf", width = 4, height=5, units="in")
Sys.sleep(5)

data <- read.csv("fastlsa_subsampling_core_subsampled_sets.csv")

# range of core size from 0.05 to 0.3 and prev from 0.1 to 1
names <- c("Input", "Degree", "Random", 
          "Degree size: 0.2 prev:0.1",  "Random size: 0.2 prev:0.1",
          "Degree size: 0.2 prev:0.5",  "Random size: 0.2 prev:0.5"
          )
data <- data[data$Network %in% names,]
data$Network <- factor(data$Network, levels=c("Input", "Degree", "Random", 
                                              "Degree size: 0.2 prev:0.1", "Random size: 0.2 prev:0.1",
                                              "Degree size: 0.2 prev:0.5", "Random size: 0.2 prev:0.5"))
sets <- c('Difference', 'Intersection 0.1', 'Intersection 0.2', 'Intersection 0.5', 'Intersection 1')

difference <- data[data$Set.type %in% sets[[1]],]
ggplot(difference, aes(x=Samples, y=Set.size, colour=Network)) + 
  geom_line(size=1) + geom_point(alpha=0.01) + 
  theme_minimal() + theme(axis.text.x = element_text(angle=90, hjust=1)) + scale_color_viridis_d() + 
  xlab('Number of networks') + ylab('Size of difference') 
ggsave("../Figures/gutmicrobiome_difference_samples.pdf", width = 6, height=3, units="in")
Sys.sleep(5)

# need to remove 0s for intersection
i1 <- data[data$Set.type %in% sets[2:4],]
i1 <- i1[i1$Set.size != 0,]
ggplot(i1, aes(x=Samples, y=Set.size, colour=Network)) + #scale_x_continuous(limits=c(15,20)) + 
  geom_point(alpha=0.01) + geom_line(size=1) + 
  facet_grid(~Set.type) + 
  theme_minimal() + theme(axis.text.x = element_text(angle=90, hjust=1), legend.position="none") + scale_color_viridis_d() + 
  xlab('Number of networks') + ylab('Size of intersection') 
ggsave("../Figures/gutmicrobiome_intersections_samples.pdf", width = 6, height=3, units="in")
Sys.sleep(5)

data <- read.csv("fastlsa_core_centralities.csv", stringsAsFactors = FALSE)
data$Network <- factor(data$Network, levels=c("Input", "Degree", "Random",
                                              "Degree \nsize: 0.2 \nprev:0.1\n", "Random \nsize: 0.2 \nprev:0.1\n"))
data <- data[data$Network %in% names,]

library(ggrepel)
data <- data[data$Centrality %in% c('Betweenness'),]
data <- data[data$Network %in% c('Input', 'Random', 'Degree'),]
stats <- read.csv("fastlsa_core_centrality_stats.csv", stringsAsFactors=FALSE)
stats <- stats[stats$Comparison %in% c('Random', 'Degree'),]
stats <- stats[stats$P < 0.15,]
data <- data[data$Node %in% stats$Node,]
# set data$tax to NA for random
data$Node[data$Node == 'NA'] <- 'Bin'
# remove prefix for taxa in data
data$Node[data$Node == 'g_Dorea'] = 'Dorea'
data$Node[data$Node == 'uc_f_Ruminococcaceae'] = 'Ruminococcaceae'
data$Node[data$Node == 'uc_o_Clostridiales'] = 'Clostridiales'
data$Node[data$Node == 'g_Blautia'] = 'Blautia'
data$Label <- data$Node
data$Label[data$Network == 'Random'] <- NA
data$Label[data$Network == 'Degree'] <- NA

ggplot(data, aes(x=Lower.limit, y=Upper.limit, colour=Network)) + 
  geom_point(alpha=0.5, size=2) + facet_grid(~Node) + 
  scale_y_continuous(limits=c(0.3, 1)) + scale_x_continuous(limits=c(0, 1)) + 
  theme_minimal() + theme(axis.text.x = element_text(angle=90, hjust=1)) + scale_color_viridis_d() + 
  geom_label_repel(label=data$Label, nudge_x=0.01, nudge_y=-0.2, size=2.9) + 
  xlab("Lower limit of 95% confidence interval") + ylab("Upper limit") + theme(strip.text.x = element_blank())
ggsave("../Figures/gutmicrobiome_topcentralities.pdf", width = 6, height=4, units="in")
Sys.sleep(5)

# add metadata for the cytoscape figure
network <- read.graph('gutmicrobiome20.graphml', format='graphml')
tax <- get.vertex.attribute(network, 'name')
no_prefix <- c()
for (name in tax){
  new_name <- strsplit(as.character(name), split='_')
  if (length(new_name[[1]]) == 3){
    if (nchar(new_name[[1]][3]) < 5){
      no_prefix <- c(no_prefix, paste(new_name[[1]][length(new_name[[1]])-1], new_name[[1]][length(new_name[[1]])], sep='_'))
    } else {
      no_prefix <- c(no_prefix, new_name[[1]][length(new_name[[1]])])
    }
  } else {
    no_prefix <- c(no_prefix, new_name[[1]][length(new_name[[1]])])
  }
}
data <- data.frame(key=tax, tax=no_prefix)


clusters <- igraph::cluster_walktrap(network)
data$clusters <- c()
for (i in 1:length(clusters$names)){
  name <- clusters$names[[i]]
  data$clusters[data$key == name] <- clusters$membership[[i]]
}

# first get list of most common enterotype per participant

enterotype_list <- c()
for (i in 1:length(tabs)){
  tab <- tabs[[i]]
  enterotype_list <- c(enterotype_list, names(sort(table(tab$Enterotype_nr), decreasing=TRUE))[[1]])
}

data$median <- 0
for (i in 1:nrow(data)){
  taxon <- data$key[[i]]
  abundances <- c()
  for (j in 1:length(rmp)){
    taxon_abundances <- as.data.frame(t(seqtime::normalize(t(rmp[[j]]))))[[taxon]]
    abundances <- c(abundances, mean(taxon_abundances, na.rm=TRUE))
  }
  taxon_difference <- median(abundances[enterotype_list %in% c(2)]) - median(abundances[enterotype_list %in% c(1,3,4)])
  data$median[[i]] <- taxon_difference
}
  
write.csv(data, 'cluster_correlations.csv') 

# 1 = Bacteroides 1
# 2 = Ruminococcaceae
# 3 = Bacteroides 2
# 4 = Prevotella


