# This file contains the scripts used to simulate the effect of increasing network numbers.

import networkx as nx
from random import sample
from anuran.nulls import generate_null
from anuran.sets import generate_sample_sizes


def add_edges(network, connectivity):
    """
    Adds random edges to the network until the desired connectivity is reached.
    :param network: NetworkX network
    :param connectivity: Desired fraction of edges connected
    :param total_edges: Total edge number
    :return: Network with new edges
    """
    total_edges = len(network.nodes) * (len(network.nodes) - 1) / 2
    while len(network.edges) < total_edges * connectivity:
        new_edge = sample(range(len(network.nodes)), 2)
        if new_edge not in network.edges:
            network.add_edge(new_edge[0], new_edge[1], weight=sample([-1, 1], 1)[0])
    return network


def generate_core_edges(network, connectivity, cores, n):
    """
    Generates a list of core edges.
    :param network: NetworkX network
    :param connectivity: Desired fraction of possible edges connected
    :param cores: Dictionary with core size as keys, core prevalence as values
    :param n: number of networks
    :return: Dictionary with core edge lists
    """
    total_edges = len(network.nodes) * (len(network.nodes) - 1) / 2
    network_size = total_edges * connectivity
    core_edges = {a: [] for a in cores.keys()}
    all_edges = []
    for value in core_edges.keys():
        while len(core_edges[value]) < network_size * value:
            core_edge = sample(range(len(network.nodes)), 2)
            if core_edge not in all_edges:
                weight = sample([-1, 1], 1)[0]
                core_edges[value].append((core_edge[0], core_edge[1], weight))
                all_edges.append((core_edge[0], core_edge[1]))
    present_edges = []
    for i in range(n):
        present_edges.append([])
    # on average, edges are present in a fraction of networks
    # so the total number of edges is lower than the list
    # generated above
    for value in cores:
        for edge in core_edges[value]:
            occurrences = sample(range(n), round(cores[value] * n))
            for occurrence in occurrences:
                present_edges[occurrence].append(edge)
    return present_edges


def add_core_edges(network, core_edges):
    """
    Adds edges from core network to observed network,
    :param network: NetworkX network
    :param core_edges: Dictionary with edge lists
    :return:
    """
    for edge in core_edges:
        network.add_edge(edge[0], edge[1], weight=edge[2])
    return network


# 5, 10, 15 % of edges connected
# not the same as connectivity definition!
# 30 networks per type

# add edges based on core probability
# then add edges up to connectivity

basic_network = nx.Graph()
for i in range(50):
    basic_network.add_node(i)

n = 50
# no core
# networks with random edges
no_core = {0.05: []} #, 0.1: [], 0.15: []}
for i in range(n):
    for connectivity in no_core.keys():
        new_network = basic_network.copy(as_view=False)
        no_core[connectivity].append((i, add_edges(new_network, connectivity)))

# generate networks with cores
# 0% prevalence core at 75% of network size
# 33% prevalence core at 20% of network size
# edges may be slightly higher prevalence if inferred randomly
core_1 = {0.05: []} #, 0.1: [], 0.15: []}
cores = {0.2: 0.33}
for connectivity in core_1.keys():
    core_edges = generate_core_edges(basic_network, connectivity, cores, n)
    # with list of core edges, generate network
    for i in range(n):
        new_network = basic_network.copy(as_view=False)
        new_network = add_core_edges(new_network, core_edges[i])
        # add remaining edges based on edges still missing
        core_1[connectivity].append((i, add_edges(new_network, connectivity)))

# 0% prevalence core at 80% of network size
# 50% prevalence core at 20% of network size
core_2 = {0.05: []} #, 0.1: [], 0.15: []}
cores = {0.2: 0.5}
for connectivity in core_2.keys():
    core_edges = generate_core_edges(basic_network, connectivity, cores, n)
    # with list of core edges, generate network
    for i in range(n):
        new_network = basic_network.copy(as_view=False)
        new_network = add_core_edges(new_network, core_edges[i])
        # add remaining edges based on edges still missing
        core_2[connectivity].append((i, add_edges(new_network, connectivity)))

# 0% prevalence core at 50% of network size
# 33% prevalence core at 20% of network size
# 50% prevalence core at 30% of network size
core_3 = {0.05: []} #, 0.1: [], 0.15: []}
cores = {0.2: 0.33, 0.3: 0.5}
for connectivity in core_3.keys():
    core_edges = generate_core_edges(basic_network, connectivity, cores, n)
    # with list of core edges, generate network
    for i in range(n):
        new_network = basic_network.copy(as_view=False)
        new_network = add_core_edges(new_network, core_edges[i])
        # add remaining edges based on edges still missing
        core_3[connectivity].append((i, add_edges(new_network, connectivity)))

# 10% prevalence core at 50% of network size
# 33% prevalence core at 30% of network size
# 50% prevalence core at 20% of network size
core_4 = {0.05: []} #, 0.1: [], 0.15: []}
cores = {0.5: 0.1, 0.3: 0.33, 0.2: 0.5}
for connectivity in core_4.keys():
    core_edges = generate_core_edges(basic_network, connectivity, cores, n)
    # with list of core edges, generate network
    for i in range(n):
        new_network = basic_network.copy(as_view=False)
        new_network = add_core_edges(new_network, core_edges[i])
        # add remaining edges based on edges still missing
        core_4[connectivity].append((i, add_edges(new_network, connectivity)))

all_networks = {'no_core': {str(x): no_core[x] for x in no_core},
                'core_1': {str(x): core_1[x] for x in core_1},
                'core_2': {str(x): core_2[x] for x in core_2},
                'core_3': {str(x): core_3[x] for x in core_3},
                'core_4': {str(x): core_4[x] for x in core_4}}

for networks in all_networks:
    random_models, degree_models = generate_null(all_networks[networks], n=5, core=6, fraction=None, prev=None)

    print('Models done!')
    samples = generate_sample_sizes(all_networks[networks], random_models, degree_models,
                                    sign=False, set_operation=['intersection', 'difference'], core=6,
                                    fractions=None, perm=5, prev=None,
                                    sizes=[0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 1], limit=10,
                                    number=[5, 10, 15, 20, 25, 30, 35, 40, 45, 50])
    print(networks + ' done!')
    samples.to_csv(os.getcwd() + '//sim//' + networks + '_subsampled_sets.csv')

